/******************************************************************************
 *	File	:	main.c
 *	Version :	1.5200
 *	Date	:	2021/11/23
 *	Author	:	XFORCE
 *
 *	Demonstrate how to use ECM-XF control drive type slave device,
 *	And how to configure PDO of slave.
 *
 *
 * @copyright (C) 2021 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <stdlib.h>
#include <unistd.h>
#include "EcmUsrDriver.h"
#include "platform.h"
#include "Utility.h"
#include "PdoDefine.h"

#define TEST_SPI_FREQ			24000000
#define DC_ACTIVE_CODE			0x300
#define BASE_CYCTIME			1000000
/*
 * TEST_CYCTIME_DIVID
 * 1 : for TEST_CYCTIME_MULTI
 * 2 : 0.5ms
 * 4 : 0.25ms
 * 8 : 0.125ms
 *
 * TEST_CYCTIME_MULTI
 * 1 : for TEST_CYCTIME_DIVID
 * 2 : 2ms
 * 4 : 4ms
 * */
#define TEST_CYCTIME_DIVID		1
#define TEST_CYCTIME_MULTI		1
#define TEST_CYCTIME_NS			((BASE_CYCTIME*TEST_CYCTIME_MULTI)/TEST_CYCTIME_DIVID)
#define ONE_SEC_CYC_CNT			(1000000000/TEST_CYCTIME_NS)
 /*
  * Change this define for your need
  * */
  /* Velocity parameter*/
#define TEST_RPM				1200
#define TEST_PPR				10000
#define TEST_VEL_PLS_PER_MS		((TEST_RPM*TEST_PPR)/60000)
#define TEST_VEL_PLS_PER_CYC	((TEST_VEL_PLS_PER_MS*TEST_CYCTIME_MULTI)/TEST_CYCTIME_DIVID)
#define TEST_ACC_TIME_MS		TEST_VEL_PLS_PER_CYC
#define TEST_CONST_TIME_MS		3000
#define TEST_STOP_TIME_MS		3000

#define TEST_ACC_TIME_CNT		((TEST_ACC_TIME_MS*TEST_CYCTIME_DIVID)/TEST_CYCTIME_MULTI)
#define TEST_CONST_TIME_CNT		((TEST_CONST_TIME_MS*TEST_CYCTIME_DIVID)/TEST_CYCTIME_MULTI)
#define TEST_STOP_TIME_CNT		((TEST_STOP_TIME_MS*TEST_CYCTIME_DIVID)/TEST_CYCTIME_MULTI)
#if(TEST_CYCTIME_MULTI == 1)
#define TEST_ACC_FRAC			1
#define TEST_ACC_DENO			(TEST_ACC_TIME_CNT/TEST_VEL_PLS_PER_CYC)
#else
#define TEST_ACC_FRAC			(TEST_VEL_PLS_PER_CYC/TEST_ACC_TIME_CNT)
#define TEST_ACC_DENO			1
#endif

/* CiA402 */
#define SERVO_ON_STATE			CIA402_SW_OPERATIONENABLED
#define SERVO_OFF_STATE			CIA402_SW_READYTOSWITCHON
/* Test run time*/
#define TEST_BURNTIME_SEC		120
#define TEST_BURNTIME_CNT		(TEST_BURNTIME_SEC*(TEST_CYCTIME_DIVID*1000)/TEST_CYCTIME_MULTI)

extern uint8_t RxPDOData[TEST_SPI_DATA_SIZE];
extern uint8_t TxPDOData[TEST_SPI_DATA_SIZE];

int nPos[TEST_DRV_CNT];

int main (int argc,char ** argv)
{
	int i = 0, IoIdx = 0, AxisIdx = 0;
	int nret = 0, nRunCnt = 0;
	uint8_t u8Version = 0, u8FifoCnt = 0, u8FifoCntMax = TEST_RXFIFO_CNT;
	uint8_t u8State = 0, u8WkcErrCnt = 0, u8CrcErrCnt = 0, u8IsSlvAlive = 0;
	uint16_t u16RxPDOSize = 0, u16TxPDOSize = 0;
	int8_t SlaveCnt = 0;
	RXPDO_ST_DEF_T	*pAllDevRx = (RXPDO_ST_DEF_T *)RxPDOData;
	TXPDO_ST_DEF_T	*pAllDevTx = (TXPDO_ST_DEF_T *)TxPDOData;
	memset(RxPDOData, 0, sizeof(RxPDOData));
	memset(TxPDOData, 0, sizeof(TxPDOData));
	memset(nPos, 0, sizeof(nPos));
	//	Hardware platform dependency
	UserSys_Init(TEST_SPI_FREQ);
	printf("Enter any key after ECM link done\n");
	getchar();
	uint16_t u16SpiSize = TEST_SPI_DATA_SIZE;// Notice : this MARCO at EcmUsrDriver.h
	u8Version = ECM_InitLibrary(&u16SpiSize, ECM_CRC_TYPE_NONE);
	if (u8Version == 0) {
		printf("ECM_InitLibrary : FAILED\r\n");
		return -1;
	}
	printf("FW Ver(0x%X)\r\n", u8Version);
	nret = ECM_EcatInit(DC_ACTIVE_CODE, (BASE_CYCTIME*TEST_CYCTIME_MULTI) / TEST_CYCTIME_DIVID);
	if (nret <= 0) {
		printf("ECM_EcatInit : Failed\r\n");
		return -1;
	}
	SlaveCnt = ECM_EcatSlvCntGet();
	if (SlaveCnt < TEST_SLV_CNT) {
		printf("ERROR : Slave count %d < %d\r\n", SlaveCnt, TEST_SLV_CNT);
		return -1;
	}
	nret = ConfigDrive(1, 0, TEST_DRV_CNT-1, 0, 1, 0x1600, 0x1A00, TEST_MODEOP);
	if (nret <= 0) {
		printf("ConfigDrive Failed\r\n");
		return -1;
	}
	ECM_SetTxFIFOCnt(TEST_TXFIFO_CNT);
	ECM_SetRxFIFOCnt(TEST_RXFIFO_CNT);
	nret = ECM_CheckMEMSpace(TEST_PDO_TO_FIFO_ONCE);
	if (nret <= 0) {
		printf("ECM_CheckMEMSpace : Failed\r\n");
		return -1;
	}
	printf("ECM_CheckMEMSpace : Pass\r\n");
	ECM_InitFIFO();
	ECM_EnableFIFO(0);
	u16RxPDOSize = ECM_FifoRxPdoSizeGet();
	u16TxPDOSize = ECM_FifoTxPdoSizeGet();
	if (u16RxPDOSize != sizeof(RXPDO_ST_DEF_T)) {
		printf("RxPDO size %d != %d\r\n", u16RxPDOSize, sizeof(RXPDO_ST_DEF_T));
		return -1;
	}
	if (u16TxPDOSize != sizeof(TXPDO_ST_DEF_T)) {
		printf("TxPDO size %d != %d\r\n", u16TxPDOSize, sizeof(TXPDO_ST_DEF_T));
		return -1;
	}
	nret = SetDriverCmdOpMode(0, TEST_DRV_CNT-1, TEST_MODEOP, 1);
	
	for(AxisIdx=0; AxisIdx<TEST_DRV_CNT; AxisIdx++){
		ECM_SetPtParam(AxisIdx, 1000, 1000);
	}
	
	nret = SetStateAndCheck(0xFF, EC_STATE_SAFE_OP, 5);
	if(nret <= 0){
		return -1;
	}
	printf("=============\r\n");
	printf("SAFE-OP state\r\n");
	printf("=============\r\n");
	printf("Check DC stable : \n");
	nRunCnt = 0;
	while (1) {
		UserDelay(1000000);
		nret = ECM_CheckDCStable();
		if (nret == 1) {
			break;
		}
		if(nRunCnt>5){
			printf("By");
			break;
		}
		nRunCnt++;
	}
	printf("PASS\r\n");
	printf("=============\r\n");
	printf("TO OP state\r\n");
	printf("=============\r\n");
	nret = SetStateAndCheck(0xFF, EC_STATE_OPERATIONAL, 5);
	if (nret <= 0) {
		for(i=0;i<SlaveCnt;i++){
			ECM_SlaveInfoGet(i, ECM_SLV_INFO_ALstatuscode, (uint8_t *)&u16RxPDOSize);
			printf("(%d) ALstatuscode 0x%x\n",i,u16RxPDOSize);
		}
		return -1;
	}
	printf("Enter to ServoOn\n");
	getchar();	
	ECM_EnableFIFO(1);
	AlignmentAndWaitServoOn(nPos);
	//
	//	PDO example : use FIFO do cyclic data exchange
	printf("Enter to start application\n");
	getchar();
	printf("Start application\n");
	ECM_PT(0, 50, 1);
	ECM_PT(1, 50, 1);
	sleep(10);
	ECM_PT(0, 0, 1);
	sleep(20);
	ECM_PT(1, 0, 1);
	printf("=============\r\n");
	printf("Exit\r\n");
	printf("=============\r\n");
	UserDelay(1000000);
	nret = ECM_StateCheck(0xFF, EC_STATE_PRE_OP, 2000);
	if (nret == 0) {
		return -1;
	}
	UserSys_Close();
}
//	(C) COPYRIGHT 2020 NEXTW Technology Corp.

