/******************************************************************************
 *	File	:	platform.c
 *	Version :	0.1
 *	Date	:	2020/04/24
 *	Author	:	User
 *
 *	Hardware dependency function - C source file
 *
 *	Provide necessary function for EcmUsrDriver
 *
 * @copyright (C) 2020 NEXTW TECHNOLOGY CO., LTD.. All rights reserved.
 *
 ******************************************************************************/
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <fcntl.h>
#include <pthread.h>
#include <errno.h>
#include "platform.h"
#include "EcmDriver.h"
#include "EcmUsrDriver.h"
#include "spidev.h"

#define TRUE	1
#define FALSE	0

#define DEVNAME	"/dev/spidev0.0"

typedef unsigned char uchar;
typedef unsigned int uint;
typedef unsigned long ulong;
typedef uchar BOOL;

struct spi_ioc_transfer trans;

static int xf_handle = -1;
static uint32_t g_u32CrcType = ECM_CRC_TYPE_NONE;

/*************************************************
 * 
 * Local Bus module API 
 * 
 * **********************************************/
int XF_SetDevType(int nType){ return 1; }
int XF_GetDevType(){ return 1; }
int32_t UserGetGpioData_u32(unsigned long *pulVal){ return 0; }
int32_t UserSetGpioData_u32(unsigned long ulVal){ return 0; }
int32_t UserGetGpioDir(unsigned long *pulDir){ return 0; }
int32_t UserSetGpioDir(unsigned long ulDir){ return 0; }
int32_t UserSetGpioData(uint8_t u8Idx, unsigned long ulVal){ return 0; }
int32_t UserGetGpioData(uint8_t u8Idx, unsigned long *pulVal){ return 0; }

static int transfer(int fd, uint8_t const *tx, uint8_t const *rx, size_t len)
{
	int ret;
	int out_fd;

        struct spi_ioc_transfer tr = {0,};
	tr.tx_buf = (unsigned long)tx;
	tr.rx_buf = (unsigned long)rx;
	tr.len = len;
	tr.speed_hz = trans.speed_hz;
	tr.bits_per_word = trans.bits_per_word;
	ret = ioctl(fd, SPI_IOC_MESSAGE(1), &tr);
	if (ret < 1)
		printf("can't send spi message");
        return ret; 
}

/*************************************************
 * 
 * Polling DMA Done
 * 
 * **********************************************/

/*************************************************
 * 
 * ECM-XF Platform API 
 * 
 * **********************************************/
int UserSys_Init(uint32_t u32SpiFreq)
{
        int ret;
        uint32_t mode;
        struct sched_param param = {.sched_priority = 99};
	if(xf_handle < 0 && u32SpiFreq != 0){
		xf_handle = open( DEVNAME, O_RDONLY);
		if(xf_handle < 0){
                        printf("Init error: %d\n", xf_handle);
			return -EIO;
                }
                mode = SPI_MODE_0;
                trans.speed_hz = u32SpiFreq;
                trans.bits_per_word = 8;
	        ret = ioctl(xf_handle, SPI_IOC_WR_MODE32, &mode);
	        if (ret == -1)
		        printf("can't set spi mode");

	        /*
	         * bits per word
	         */
	        ret = ioctl(xf_handle, SPI_IOC_WR_BITS_PER_WORD, &trans.bits_per_word);
	        if (ret == -1)
		        printf("can't set bits per word");

	        /*
	         * max speed hz
	         */
	        ret = ioctl(xf_handle, SPI_IOC_WR_MAX_SPEED_HZ, &trans.speed_hz);
	        if (ret == -1)
		        printf("can't set max speed hz");
   
	}
        pthread_setschedparam( pthread_self(), SCHED_FIFO, &param);
        return 0;
}
void UserSys_Close(void)
{
	if(xf_handle >= 0)
		close(xf_handle);
	xf_handle = -1;
}
int UserSpiDataExchange(uint8_t *pTxBuf, uint8_t *pRxBuf, uint32_t u32PackSize)
{
        transfer(xf_handle, pTxBuf, pRxBuf, u32PackSize);
	return 1;
}
int UserDelay(uint32_t MircoSec)
{
	usleep(MircoSec);
	return 1;
}
// Optional function
uint32_t UserSetCrcType(uint32_t u32CrcType)
{
	g_u32CrcType = u32CrcType;
	return g_u32CrcType;
}
uint32_t UserCalCrc(uint8_t *pu8Addr, uint32_t u32Size)
{
	return ECM_CRC_MAGIC_NUM;
}
//	(C) COPYRIGHT 2020 NEXTW Technology Corp.
